/*
 * ADC.c
 *
 * Created: 18.07.2012 21:43:02
 *  Author: admin
 */ 
#include "avr/io.h"
#include "ADC.h"

volatile uint16_t ADC_Sample=0;

void ADC_Init()
{
	//Reference VCC, ADC-Ch7
	ADMUX = (0<<REFS1)|(1<<REFS0)|(0<<ADLAR)|(0<<MUX3)|(1<<MUX2)|(1<<MUX1)|(1<<MUX0);
	//ADC_SetPrescaler(eADC_PRESCALER_128);
	ADC_SetPrescaler(eADC_PRESCALER_32);
	ADC_Enable();
	
}

ADC_Init_Bandgap()
{
	//Reference internal, ADC-Ch7, ADC-Ch6
	ADMUX = (1<<REFS1)|(1<<REFS0)|(0<<ADLAR)|(0<<MUX3)|(1<<MUX2)|(1<<MUX1)|(0<<MUX0);
	ADC_SetPrescaler(eADC_PRESCALER_128);
	ADC_Enable();
}


void ADC_Enable()
{
	ADCSRA |= (1<<ADEN);
}	

void ADC_Disable()
{
	ADCSRA &= ~(1<<ADEN);
}

void ADC_EnableAutoTrigger()
{
	ADCSRA |= (1<<ADATE);
}

void ADC_DisableAutoTrigger()
{
	ADCSRA &= ~(1<<ADATE);
}

void ADC_EnableInterrupt()
{
	ADCSRA |= (1<<ADIE);
}

void ADC_DisableInterrupt()
{
	ADCSRA &= ~(1<<ADIE);
}

void ADC_SetPrescaler(eADC_PRESCALER adc_prescaler)
{
	ADCSRA |= adc_prescaler;
}

uint8_t ADC_GetPrescaler()
{
	eADC_PRESCALER adc_prescaler = ADCSRA & 0x07;
	uint8_t result;
	
	switch(adc_prescaler)
	{
		case eADC_PRESCALER_2:
			result = 2;
		break;
		case eADC_PRESCALER_4:
			result = 4;
		break;
		case eADC_PRESCALER_8:
			result = 8;
		break;
		case eADC_PRESCALER_16:
			result = 16;
		break;
		case eADC_PRESCALER_32:
			result = 32;
		break;		
		case eADC_PRESCALER_64:
			result = 64;
		break;
		case eADC_PRESCALER_128:
			result = 128;
		break;
	}
	return result;
}


void ADC_SetReference(eADC_REFERENCE adc_reference)
{
	//Wait for Conversion complete before set channel and starting new conversion
	while((ADCSRA&(1<<ADSC)))asm("nop");
	ADMUX &= ~((1<<REFS1)|(1<<REFS0));
	ADMUX |= adc_reference;
}

eADC_REFERENCE ADC_GetReference()
{
	return (ADMUX & ((1<<REFS1)|(1<<REFS0)));
}

uint16_t ADC_SampleChannel(eADC_CHANNEL adc_channel)
{
	//Wait for Conversion complete before set channel and starting new conversion
	while((ADCSRA&(1<<ADSC)))asm("nop");
	//Set channel
	ADMUX &= ~((1<<MUX3)|(1<<MUX2)|(1<<MUX1)|(1<<MUX0));
	ADMUX |= adc_channel;
	//start adc conversion
	ADC_StartConversion();
	//Wait for completion
	while((ADCSRA&(1<<ADSC)))asm("nop");
	//Get result
	return(ADC);
}

void ADC_SetChannel(eADC_CHANNEL adc_channel)
{
	//Wait for Conversion complete before set channel and starting new conversion
	while((ADCSRA&(1<<ADSC)))asm("nop");
	//Set channel
	ADMUX &= ~((1<<MUX3)|(1<<MUX2)|(1<<MUX1)|(1<<MUX0));
	ADMUX |= adc_channel;}

uint16_t ADC_CH6()
{
	//start adc conversion
	ADC_StartConversion();
	//Wait for completion
	while((ADCSRA&(1<<ADSC)))asm("nop");
	//Get result
	return(ADC*12);

}

