/*
 * Goertzel.c
 *
 * Created: 18.07.2012 21:32:25
 *  Author: admin
 */ 

#include "Goertzel.h"
#include <math.h>

extern uint16_t AsmLogMagnitude(uint16_t);

volatile int16_t Q0=0;
volatile int16_t Q1=0;
volatile int16_t Q2=0;
volatile int16_t coeff=COSINE;
volatile int16_t ADCSample=0;
volatile int32_t AC0=0;
volatile int32_t AC1=0;
volatile uint8_t Goertzel_Enable=0;
volatile uint8_t Goertzel_Count=0;
volatile uint16_t Goertzel_F100=0;
volatile uint8_t Goertzel_Complete=0;
//volatile uint16_t GoertzelBuffer[NUMBER_OF_SAMPLES];
volatile int8_t GoertzelBuffer[NUMBER_OF_SAMPLES]; //ok
volatile uint8_t GoertzelBufferFilled=0;
volatile uint8_t gain_calc;
volatile int16_t x;
volatile int16_t high_x;
volatile int16_t low_x;
volatile int16_t gain_cnt;
volatile int16_t gain;
// The current sample, in two formats -- each convenient for a purpose
// As signed 16 bit, for Goertzel algorithm
volatile int16_t sample16;
// As unsigned 8 bit, for playback and DTMF detection
volatile uint8_t sample8;

uint16_t* GoertzelFillBuffer(uint16_t value)
{
	uint8_t i=0;
	GoertzelBuffer[i]=value;
	i++;
}

int16_t GoertzelFilter(int16_t cos_coef, uint16_t *pValueArray)
{
	for(uint8_t i=0;i<NUMBER_OF_SAMPLES;i++)
	{
		Q0 = *pValueArray>> 8;
		pValueArray++;
		AC0 = (int32_t) Q1*cos_coef;
		AC0 >>= 14;
		AC0 -= Q2;
		Q0 += AC0;
		Q2 = Q1;
		Q1 = Q0;
	}	
	AC0 = (int32_t) Q1 * Q1;
	AC0 += (int32_t) Q2 * Q2;
	AC1 = (int32_t) Q1 * cos_coef;
	AC1 >>= 14;
	AC1 *= Q2;
	AC0 -= AC1;
	Q1=0;
	Q2=0;
	return (int16_t)(AC0 >> 14);	
}	

uint16_t GoertzelFilter1(int16_t cos_coeff, int8_t *pValueArray)
{
	int16_t d1=0,d2=0,mag=0,y=0;
	for (uint8_t i=0;i<NUMBER_OF_SAMPLES;i++)
	{
		y=((d1*cos_coeff)>>8)-d2+GoertzelBuffer[i];//*(pValueArray+i);//ss8;
		d2=d1;
		d1=y;
	}
	d1=d1>>8;
	d2=d2>>8;
	mag=d1*d1+d2*d2-d1*((d2*cos_coeff)>>8);
	d1=d2=0;
	
	return 10*log( sqrt(mag) );
 	//return 10*AsmLogMagnitude(mag);
}

int16_t GoertzelGainControl(uint16_t *pValueArray)
{
	uint16_t value;
	for (gain_cnt=0;gain_cnt<NUMBER_OF_SAMPLES;gain_cnt++)
	{
		x=*(pValueArray+gain_cnt);
		if(x > high_x) high_x = x;                // Find maximum value
		else if(x < low_x) low_x = x;             // Find minimum value		
	}            
	// compute gain
	gain = ( 1023 / (high_x - low_x) );
	// low_x will contain the average value
	low_x += (high_x - low_x)>>1;
	
	//Scale Values
	for (gain_cnt=0;gain_cnt<NUMBER_OF_SAMPLES;gain_cnt++)
	{
		x=*(pValueArray+gain_cnt);
		x = (x - low_x) * gain; 
		*(pValueArray+gain_cnt)=x;
	}
	return gain;
}	

int16_t GoertzelOutput()
{
	AC0 = (int32_t) Q1 * Q1;
	AC0 += (int32_t) Q2 * Q2;
	AC1 = (int32_t) Q1 * coeff;
	AC1 >>= 14;
	AC1 *= Q2;
	AC0 -= AC1;
	Q1=0;
	Q2=0;
	return (int16_t)(AC0 >> 14);
}

