/*
 * Goertzel.h
 *
 * Created: 18.07.2012 21:32:38
 *  Author: admin
 */ 


#ifndef GOERTZEL_H_
#define GOERTZEL_H_

#include "Goertzel.h"
#include <avr/io.h>
#include <math.h>

#define TWO_PI 6.283185307179586476925286766559f
#define COS_SCALE        7
#define Sample_Frequency 8000.0 //4 kHz
#define Frequency_40     40.0  //Search frequency 40Hz
#define Frequency_63     63.0  //Search frequency 63Hz
#define Frequency_98     98.0  //Search frequency 98Hz
#define Frequency_160    160.0  //Search frequency 160Hz
#define Frequency_250    250.0  //Search frequency 250Hz
#define Frequency_400    400.0  //Search frequency 400Hz
#define Frequency_625    625.0  //Search frequency 625Hz
#define Frequency_1000   1000.0  //Search frequency 1000Hz
#define Frequency_1500   1500.0  //Search frequency 1500Hz
#define Frequency_1800   1800.0  //Search frequency 1800Hz
#define Frequency_2000   2000.0  //Search frequency 2000Hz
#define Frequency_2500   2500.0  //Search frequency 2500Hz
#define Frequency_3000   3000.0  //Search frequency 3000Hz
#define Frequency_3200   3200.0  //Search frequency 3200Hz
#define Frequency_3800   3800.0  //Search frequency 3800Hz
#define f_40     (int)(0.5+Frequency_40*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_63     (int)(0.5+Frequency_63*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_98     (int)(0.5+Frequency_98*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_160    (int)(0.5+Frequency_160*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_250    (int)(0.5+Frequency_250*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_400    (int)(0.5+Frequency_400*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_625    (int)(0.5+Frequency_625*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_1000   (int)(0.5+Frequency_1000*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_1500   (int)(0.5+Frequency_1500*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_1800   (int)(0.5+Frequency_1800*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_2000   (int)(0.5+Frequency_2000*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_2500   (int)(0.5+Frequency_2500*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_3000   (int)(0.5+Frequency_3000*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_3200   (int)(0.5+Frequency_3200*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define f_3800   (int)(0.5+Frequency_3800*(float)NUMBER_OF_SAMPLES/Sample_Frequency)
#define COSINE_40   (int16_t)(cos(TWO_PI*(float)f_40/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_63   (int16_t)(cos(TWO_PI*(float)f_63/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_98   (int16_t)(cos(TWO_PI*(float)f_98/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_160  (int16_t)(cos(TWO_PI*(float)f_160/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_250  (int16_t)(cos(TWO_PI*(float)f_250/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_400  (int16_t)(cos(TWO_PI*(float)f_400/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_625  (int16_t)(cos(TWO_PI*(float)f_625/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_1000 (int16_t)(cos(TWO_PI*(float)f_1000/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_1500 (int16_t)(cos(TWO_PI*(float)f_1500/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_1800 (int16_t)(cos(TWO_PI*(float)f_1800/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_2000 (int16_t)(cos(TWO_PI*(float)f_2000/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_2500 (int16_t)(cos(TWO_PI*(float)f_2500/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_3000 (int16_t)(cos(TWO_PI*(float)f_3000/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_3200 (int16_t)(cos(TWO_PI*(float)f_3200/(float)NUMBER_OF_SAMPLES)*256)
#define COSINE_3800 (int16_t)(cos(TWO_PI*(float)f_3800/(float)NUMBER_OF_SAMPLES)*256)

#define COSINE (1.902*(1<<COS_SCALE))  //= 2*cos(2*pi*F_search/F_sample) * (1<<COS_SCALE)
#define NUMBER_OF_SAMPLES 200 // Samples (Blocksize) for Goertzel filter
#define GOERTZEL_GAIN NUMBER_OF_SAMPLES/2

uint16_t* GoertzelFillBuffer(uint16_t value);
int16_t GoertzelFilter(int16_t cos_coef, uint16_t *pValueArray);
uint16_t GoertzelFilter1(int16_t cos_coeff, int8_t *pValueArray);
int16_t GoertzelOutput();
int16_t GoertzelGainControl(uint16_t *pValueArray);

volatile extern uint8_t Goertzel_Enable;
volatile extern uint8_t Goertzel_Count;
volatile extern uint16_t Goertzel_F100;
volatile extern uint8_t Goertzel_Complete;
volatile extern int16_t ADCSample;
volatile extern int8_t GoertzelBuffer[NUMBER_OF_SAMPLES];
//volatile extern uint16_t GoertzelBuffer[NUMBER_OF_SAMPLES];
volatile extern uint8_t GoertzelBufferFilled;
volatile extern uint8_t gain_calc;
volatile extern int16_t x; 
volatile extern int16_t high_x;
volatile extern int16_t low_x;
volatile extern int16_t gain_cnt;
volatile extern int16_t gain;
// The current sample, in two formats -- each convenient for a purpose
// As signed 16 bit, for Goertzel algorithm
volatile extern int16_t sample16;
// As unsigned 8 bit, for playback and DTMF detection
volatile extern uint8_t sample8;

#endif /* GOERTZEL_H_ */