/*
 * MMA7455L.c
 *
 * Created: 20.10.2012 17:05:17
 *  Author: admin
 */ 

#include <avr/io.h>
#include "MMA7455L.h"
#include "SPI.h"
#include <avr/interrupt.h>

volatile uint8_t MMA7455L_INT1_DRDY = 0 ;
volatile uint8_t Xdata,Ydata,Zdata;
volatile int8_t X_latch,Y_latch,Z_latch;

volatile int8_t X_BUFF[10];
volatile int8_t Y_BUFF[10];
volatile int8_t Z_BUFF[10];
volatile int8_t X_AVG=0;
volatile int8_t Y_AVG=0;
volatile int8_t Z_AVG=0;
volatile int16_t X_SUM;
volatile int16_t Y_SUM;
volatile int16_t Z_SUM;

//////////////////////////////////////////////////////////////////////////
//	Function: 	Initialize MMA7455L
//	Parameters: None
//	Return:		None
//
//	Note:       Initialize Pins, SPI and external Interrupts only
//              
//              
//////////////////////////////////////////////////////////////////////////
void MMA7455L_Init()
{
	
	//Set PD5, PD6, PD7 as input, because software spi is not used
	DDRD &= ~(1<<PIND5 | 1<<PIND6 | 1<<PIND7);
	DDRB &= ~(1<<PINB0 | 1<<PINB1) ;
	
	//Set Slave Select High to unselect MMA7455
	SPI_SetSS();
	
	//Init SPI 
	if(SPI_IsInit()==0)SPI_Init();
	
	//Configure external pin interrupts
	PORTB &= ~(1<<PINB1);
	//Enable pin change interrupt 0 (PB0..PB7)
	//PCICR |= 1<<PCIE0;
	//Enable change interrupt on pin PB1
	//PCMSK0 |= 1<<PCINT1;

}

//////////////////////////////////////////////////////////////////////////
//	Function: 	Read register content from MMA7455L via SPI
//	Parameters: Register number
//	Return:		Register content
//
//	Note:       Only read 8 bit! 
//              if SPI-Bus is in use nothing will written
//
//////////////////////////////////////////////////////////////////////////
uint8_t MMA7455L_SPI_Read(uint8_t reg)
{
	uint8_t temp;
	if (SPI_Used==0)
	{
		SPI_Used=1;
		if(reg>MM7455L_MAX_ADDR) reg=MM7455L_MAX_ADDR;
		//shift adress
		reg = reg << 1;
		//clear bit7 for read
		reg &= ~(1<<MM7455L_RW);
		//Select Device
		SPI_ClearSS();
		//Send register to read
		SPI_Write(reg);
		//Send dummy byte for get data from register
		temp=SPI_Write(MMA7455L_SPI_DUMMY);
		SPI_SetSS();
		SPI_Used=0;
		return temp;		
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////////
//	Function: 	Write content to MMA7455L-register via SPI in 3 Wire or 4 Wire mode
//	Parameters: Register number, data to write
//	Return:		None
//
//	Note:       Only write 8 bit!
//              if SPI-Bus is in use nothing will written
//
//////////////////////////////////////////////////////////////////////////
void MMA7455L_SPI_Write(uint8_t reg, uint8_t data)
{
	uint8_t temp;
	if(SPI_Used==0)
	{
		SPI_Used=1;
		if(reg>MM7455L_MAX_ADDR) reg=MM7455L_MAX_ADDR;
		//shift adress
		reg = reg << 1;
		//set bit7 for write
		reg |= (1<<MM7455L_RW);	
		SPI_ClearSS();
		//Send register to read
		SPI_Write(reg);
		//Send data
		SPI_Write(data);
		SPI_SetSS();
		SPI_Used=0;		
	}

}

//////////////////////////////////////////////////////////////////////////
//	Function: 	Write content to MMA7455L-register via SPI in 3 Wire or 4 Wire mode
//	Parameters: Register number, data to write
//	Return:		None
//
//	Note:       Only write 8 bit!
//              The use of bus will not checked by this function
//
//////////////////////////////////////////////////////////////////////////
void MMA7455L_SPI_Write1(uint8_t reg, uint8_t data)
{
	uint8_t temp;

		if(reg>MM7455L_MAX_ADDR) reg=MM7455L_MAX_ADDR;
		//shift adress
		reg = reg << 1;
		//set bit7 for write
		reg |= (1<<MM7455L_RW);
		SPI_ClearSS();
		//Send register to read
		SPI_Write(reg);
		//Send data
		SPI_Write(data);
		SPI_SetSS();

}

//////////////////////////////////////////////////////////////////////////
//	Function: 	AutoCalibration_Ion
//	Parameters: intervall_max: upper limit for zero range
//              intervall_min: lower limit for zero range
//	Return:		  Calibration
//
//	Note:       No data retention in Ion MMA7455L at power off
//              requiring to recalibrate or reload calibration
//              information
//////////////////////////////////////////////////////////////////////////
uint8_t MMA7455L_AutoCalibration(uint8_t intervall_max, uint8_t intervall_min, char direction_XYZ)
{
	int16_t tmpcalib[3];
	int16_t tmpaccel[3];
	int16_t tmpbuff[3];
	uint8_t cnt_runs =0;
	uint8_t res = 0;
	uint8_t i,j;
	int8_t zero_g=0;

	for(i=0;i<3;i++)
	{
		tmpcalib[i] = 0;
	}
	
	//SetUp_GselectIon(ION_G2);
	switch(direction_XYZ)
	{
		case 'x':
		case 'X':
			zero_g=0;
			break;
		case 'y':
		case 'Y':
			zero_g=1;
			break;	
		case 'z':
		case 'Z':
			zero_g=2;
			break;
	}
	
	while((cnt_runs++ < MAX_CNT_RUNS) && (res == 0))
	{
		for(i=0;i<3;i++) tmpaccel[i] = 0; // Clear
		
		
		
		for(i=0;i<CAL_AVG_RUNS;i++)
		{
			
				while(!(PINB & (1<<PINB1))) asm("nop");
				tmpbuff[0]=(int8_t)MMA7455L_SPI_Read(MM7455L_REG_XOUT8); //Read X,Y,Z outputs from Sensor
				tmpbuff[1]=(int8_t)MMA7455L_SPI_Read(MM7455L_REG_YOUT8);
				tmpbuff[2]=(int8_t)MMA7455L_SPI_Read(MM7455L_REG_ZOUT8);
							
			//while(!(INT1)); // wait for valid new data
			//tmpbuff[0] = MMA7455L_SPI_Read(MM7455L_REG_XOUT8);
			//
			//while(!(INT1)); // wait for valid new data
			//tmpbuff[1] = MMA7455L_SPI_Read(MM7455L_REG_YOUT8);
			//
			//while(!(INT1)); // wait for valid new data
			//tmpbuff[2] = MMA7455L_SPI_Read(MM7455L_REG_ZOUT8);

			for(j=0;j<3;j++) tmpaccel[j] += tmpbuff[j];
		}
		
		for(i=0;i<3;i++) tmpaccel[i] /= CAL_AVG_RUNS;
		
		tmpaccel[zero_g] -= 0x3F;   //Sensitivity Z axis Ion adjustment
		// 0x3F is 1g at 2g range
		for(i=0;i<3;i++)
		{
			//if(tmpaccel[i] == 0) res++;
			if((tmpaccel[i]<=intervall_max) && (tmpaccel[i]>=-intervall_min)) res++;
		}
		
		if(res >= 3)
		{
			res = 1;
			for(j=0;j<3;j++)
			tmpaccel[j] += 1;
		}
		else res = 0;
		
		for(i=0;i<3;i++) tmpcalib[i] -= tmpaccel[i];
		
		for(i=0;i<3;i++)
		{
			MMA7455L_SPI_Write(MM7455L_REG_XOFFL + i*2, (uint8_t) (((uint16_t)tmpcalib[i]) % 256));
			MMA7455L_SPI_Write(MM7455L_REG_XOFFH + i*2, (uint8_t) (((uint16_t)tmpcalib[i]) / 256));
			//calib[i] = tmpcalib[i];
		}
		
	}
	
	//FlashErase1((byte*)calib_eeprom);  // Erase block of 512 Bytes - EEPROM segment
	//CopyRam2FlashBurst((byte*)calib_eeprom, (byte*)tmpcalib, 6);   // destination adress, source address, length is 6 (important)
	
	return(res);
}

//////////////////////////////////////////////////////////////////////////
//	Function: 	Add a new value to ring buffer of x-, y-, z-axis 
//              and update average values of x-, y-, z- axis
//	Parameters: None
//	Return:		0
//
//	Note:       
//              
//              
//////////////////////////////////////////////////////////////////////////
int8_t MMA7455L_AddToAvg(uint8_t value)
{
	
	int8_t X,Y,Z;

	static uint8_t index=0;
	
			
		X=(int8_t)MMA7455L_SPI_Read(0x06); //Read X,Y,Z outputs from Sensor
		Y=(int8_t)MMA7455L_SPI_Read(0x07);
		Z=(int8_t)MMA7455L_SPI_Read(0x08);
		
		X_SUM -= X_BUFF[index];
		X_SUM += X;
		X_BUFF[index]=X;
		//Calculate average
		X_AVG = X_SUM >> MMA7455L_ACC_AVG;

		Y_SUM -= Y_BUFF[index];
		Y_SUM += Y;
		Y_BUFF[index]=Y;
		//Calculate average
		Y_AVG = Y_SUM >> MMA7455L_ACC_AVG;
		
		Z_SUM -= Z_BUFF[index];
		Z_SUM += Z;
		Z_BUFF[index]=Z;
		index++;
		//Calculate average
		Z_AVG = Z_SUM >> MMA7455L_ACC_AVG;
					
		index %= (1<<MMA7455L_ACC_AVG); //8 values
		MMA7455L_INT1_DRDY=0;

	return 0;
}

int8_t MMA7455L_GetXAvg()
{
	return X_AVG;
}

int8_t MMA7455L_GetYAvg()
{
	return Y_AVG;
}

int8_t MMA7455L_GetZAvg()
{
	return Z_AVG;
}

uint8_t MMA7455L_FillAvgBuffer()
{
	uint16_t cnt=65000;
	
	for(uint8_t i=0;i<=CAL_AVG_RUNS;i++)
	{
		while(!(PINB & (1<<PINB1)) && cnt--) asm("nop");
		MMA7455L_AddToAvg(0);
	}
}


//External Pin change interrupt on Pin PB0..PB7
ISR(PCINT0_vect)
{
	if((PINB & (1<<PINB1)))
	{
		Xdata=MMA7455L_SPI_Read(0x06); //Read X,Y,Z outputs from Sensor
		Ydata=MMA7455L_SPI_Read(0x07);
		Zdata=MMA7455L_SPI_Read(0x08);
		if(MMA7455L_INT1_DRDY==0)
		{
			X_latch=(int8_t)Xdata;
			Y_latch=(int8_t)Ydata;
			Z_latch=(int8_t)Zdata;
			MMA7455L_INT1_DRDY=1;
		}	
		MMA7455L_SPI_Write(0x17,0x03);		
		MMA7455L_SPI_Write(0x17,0x00);
	}
}