#include "SPI.h"
#include "TLC5947.h"
#include <math.h>


#define TLC5947_MAX_GREYSCALE 4095
#define TLC5947_HARDWARE_SPI

uint16_t LEDChannels[(NUM_TLC5947*24)];
unsigned char brightnessShift= 0;



//////////////////////////////////////////////////////////////////////////
//	Function  : Setup the TLC-Chip and Ball
//	Parameters: grayscale: grayscale after initialization
//	Return:		None
//
//	Note:       The PCB canvas are written clock wise and from bottom to top
//
//////////////////////////////////////////////////////////////////////////
void LED_Init(uint16_t initial_grayscale)
{
	
	// Setup TLC5947 pins
	
	#ifdef TLC5947_HARDWARE_SPI  //Hardware SPI Initialization
	DATDDR	|= ( BLANK_PIN | XLAT_PIN );
	DATPORT	|= ( BLANK_PIN | XLAT_PIN );
	DATPORT &= ~(LATPIN);
	//DATPORT &= ~(BLANK_PIN);
	//Set Blank Pin high to avoid random values during initialization
	Blank_ON();
	//Init Hardware SPI interface
	if(SPI_IsInit()==0)	SPI_Init();
	#else  //Software SPI initialization
	DATDDR	|= ( SIN_PIN |SCLK_PIN|BLANK_PIN | XLAT_PIN );
	DATPORT	|= ( SIN_PIN |SCLK_PIN|BLANK_PIN | XLAT_PIN );
	DATPORT &= ~(LATPIN);
	DATPORT &= ~(BLANK_PIN);
	#endif


	// setup button and charging indicator
	//DDRC &=~(_BV(PC4)| _BV(PC5));
	//PORTC |= _BV(PC4);
	//PORTC |= _BV(PC5);

	// turn off ADC
	ADCSRA = 0;

	// full bright
	brightnessShift = 0;
	
	//Set Grayscale to initialize values
	//Write two time, because one write didn't work.
	Clear(initial_grayscale);
	WriteLEDArray1(NUM_TLC5947,1);
	//WriteLEDArray(NUM_TLC5947);
	Clear(initial_grayscale);
	WriteLEDArray1(NUM_TLC5947,1);
	//WriteLEDArray(NUM_TLC5947);

	#if 0
	//set_sleep_mode (SLEEP_MODE_PWR_DOWN);
	//sleep_enable();
	//
	//// turn off brown-out enable in software
	//MCUCR = _BV (BODS) | _BV (BODSE);
	//MCUCR = _BV (BODS);
	//sleep_cpu ();
	//
	//// cancel sleep as a precaution
	//sleep_disable();
	#endif
}

//////////////////////////////////////////////////////////////////////////
//	Function: 	SetPoint sets a point at (column/row) in the ball. The
//              value is set in the led-array only and not shifted to the TLC     
//	Parameters: column     : column in the ball (counted clock wise from master)
//              row_led    : number of led in the column (counted from bottom)
//              gray_scale : brightness of the led (0..4095)
//	Return:		None
//
//	Note:       
//
//////////////////////////////////////////////////////////////////////////
void SetPoint( uint8_t column, uint8_t row_led,uint16_t gray_scale)
{
	LEDChannels[ (SUPERBALL_LED_PER_COLUMN*column)+row_led ] = gray_scale;
}

//////////////////////////////////////////////////////////////////////////
//	Function: 	SetPoint1 sets a point at (column/row)led in the ball and
//              shifted the value to the TLC.
//	Parameters: column      : column in the ball (counted clock wise from master)
//              row_led     : number of led in the column (counted from bottom)
//              gray_scale  : brightness of the led (0..4095)
//              clear_on    :clears the ball-array before writing values
//              enable_write: shifted out the ball-array to Hardware
//	Return:		None
//
//	Note:
//
//////////////////////////////////////////////////////////////////////////
void SetPoint1 (uint8_t column, uint8_t row_led, uint16_t gray_scale, uint8_t clear_on, uint8_t enable_write)
{
	if(clear_on)Clear(0);
	LEDChannels[ (SUPERBALL_LED_PER_COLUMN*column)+row_led ] = gray_scale;
	if(enable_write)WriteLEDArray(NUM_TLC5947);
}	

//////////////////////////////////////////////////////////////////////////
//	Function: 	Sets a number of columns of the ball on (brightness=0..4095) and write
//              the values to the TLC-Chip.
//	Parameters: column      : column in the ball (counted clock wise from master)
//              gray_scale  : brightness of the led (0..4095)
//              clear_on    :clears the ball-array before writing values
//	Return:		None
//
//	Note:
//
//////////////////////////////////////////////////////////////////////////
void SetColumnFullOn(uint8_t column, uint16_t gray_scale, uint8_t clear_on, uint8_t enable_write)
{
	if(clear_on)Clear(0);
	for(uint8_t i=0;i<SUPERBALL_LED_PER_COLUMN;i++)
	{
		//SetColumnLedOn(column,i,gray_scale,clear_on,1);
		LEDChannels[  column * SUPERBALL_LED_PER_COLUMN + i ] = gray_scale;
	}		
	if(enable_write)WriteLEDArray(NUM_TLC5947);
}


//////////////////////////////////////////////////////////////////////////
//	Function: 	sets a defines number of leds in a column on
//	Parameters: column      : column in the ball (counted clock wise from master)
//              led_start   : number of start led (counted from bottom)
//              led_end     : number of end led   (counted from bottom)
//              gray_scale  : brightness of the led (0..4095)
//              clear_on    :clears the ball-array before writing values
//              enable_write: shifted out the ball-array to Hardware
//	Return:		None
//
//	Note:
//
//////////////////////////////////////////////////////////////////////////
void SetColumnLedRangeOn(uint8_t column, uint8_t led_start, uint8_t led_end, uint16_t gray_scale, uint8_t clear_on, uint8_t enable_write)
{
	//if(clear_on)Clear(0);
	if(clear_on)
	{
		for(unsigned short LEDindex = 0; LEDindex < 384; LEDindex++)
		{
		  LEDChannels[LEDindex] = 0;
		}
	}	
	
	//for(uint8_t i=led_start;i<(led_end+1);i++)SetColumnLedOn(column,i,gray_scale,0,0);
	//memset(&LEDChannels[column * SUPERBALL_LED_PER_COLUMN + led_start],gray_scale,column * SUPERBALL_LED_PER_COLUMN + led_end);	
	for(uint8_t i=led_start;
	i<(led_end+1);
	i++)
	{
		LEDChannels[  column * SUPERBALL_LED_PER_COLUMN + i] = gray_scale;;
	}
		
	if(enable_write)WriteLEDArray(NUM_TLC5947);
}

//////////////////////////////////////////////////////////////////////////
//	Function: 	Sets a row of the ball on (brightness=0..4095) and shifted the values
//              the TLC-Chip
//	Parameters: row         : column in the ball (counted clock wise from master)
//              gray_scale  : brightness of the led (0..4095)
//              clear_on    :clears the ball-array before writing values
//	Return:		None
//
//	Note:
//
//////////////////////////////////////////////////////////////////////////
void SetRowFullOn(uint8_t row, uint16_t gray_scale, uint8_t clear_on, uint8_t enable_write)
{
	
	if(clear_on)Clear(0);
	for(uint8_t i=0;i<SUPERBALL_COLUMNS;i++)
	{
		//SetColumnLedOn(column,i,gray_scale,clear_on,1);
		LEDChannels[  i * SUPERBALL_LED_PER_COLUMN + row ] = gray_scale;
	}
	if(enable_write)WriteLEDArray(NUM_TLC5947);	
	
	//for(uint8_t i=0;i<SUPERBALL_COLUMNS;i++)SetRowLedOn(row,i,gray_scale,clear_on,0);
	//WriteLEDArray(NUM_TLC5947);
}

//////////////////////////////////////////////////////////////////////////
//	Function: 	Sets a single led in a column to the brightness
//	Parameters: column      : column in the ball (counted clock wise from master)
//              led         : number of led counted from bottom.
//              gray_scale  : brightness of the led (0..4095)
//              clear_on    : clears the ball-array before writing values
//              enable_write: writes the led-values to the hardware tlc-chip
//	Return:		None
//
//	Note:
//
//////////////////////////////////////////////////////////////////////////
void SetColumnLedOn(uint8_t column, uint8_t led, uint16_t gray_scale, uint8_t clear_on, uint8_t enable_write)
{
	if(clear_on)Clear(0);
	
	LEDChannels[  column * SUPERBALL_LED_PER_COLUMN + led ] = gray_scale;
	if(enable_write)WriteLEDArray(NUM_TLC5947);
}

//////////////////////////////////////////////////////////////////////////
//	Function: 	Sets a single led in a row to the brightness
//	Parameters: row         : row in the ball (counted from bottom of the ball)
//              led         : number of led counted from bottom.
//              gray_scale  : brightness of the led (0..4095)
//              clear_on    : clears the ball-array before writing values
//              enable_write: writes the led-values to the hardware tlc-chip
//	Return:		None
//
//	Note:
//
//////////////////////////////////////////////////////////////////////////
void SetRowLedOn(uint8_t row, uint8_t led, uint16_t gray_scale, uint8_t clear_on, uint8_t enable_write)
{
	if(clear_on)Clear(0);
	
	LEDChannels[row + SUPERBALL_LED_PER_COLUMN*led ] = gray_scale;
	if(enable_write)WriteLEDArray(NUM_TLC5947);
}


//////////////////////////////////////////////////////////////////////////
//	Function: 	Sets the internal array for storing the led values
//	Parameters: value (0..4095)
//	Return:		None
//
//	Note:
//
//////////////////////////////////////////////////////////////////////////
void Clear(uint16_t val)
{
	//brightness=val;
	//memset(&LEDChannels[0],val,NUM_TLC5947  * 24);
    for(unsigned short LEDindex = 0; LEDindex < (NUM_TLC5947  * 24); LEDindex++)
    {
	    LEDChannels[LEDindex] = val;
    }
	
}


//////////////////////////////////////////////////////////////////////////
//	Function: 	WriteLEDArray
//	Parameters: Number of canvas PCBs to write
//	Return:		None
//
//	Note:       The PCB canvas are written clock wise and from bottom to top
//
//////////////////////////////////////////////////////////////////////////
void WriteLEDArray(uint8_t count)
{
	int16_t i=0;

	if(SPI_Used==0)
	{
		SPI_Used =1;	
	
		#ifdef TLC5947_HARDWARE_SPI  //Hardware SPI
			uint8_t byte1;
			for (i = (24*count-1); i>0 ; i -= 2)
			{
				SPDR = LEDChannels[i] >> 4;
				byte1=((LEDChannels[i] & 0x000f) << 4) | (LEDChannels[i-1]>> 8);
				while(!(SPSR &(1<<SPIF)));
				SPDR= byte1;
				byte1= (uint8_t)LEDChannels[i-1];
				while(!(SPSR &(1<<SPIF)));
				SPDR = byte1;
				while(!(SPSR &(1<<SPIF)));
			}
		#else //Software SPI
			uint16_t tempOne = 0;
			for(i = (count*24);i>0;i--)
			{
				tempOne = LEDChannels[i-1];
				for (int j = 0; j < 12; j++)
				{
					if ((tempOne >> (11 - j)) & 1) DATPORT |= (1 << DATPIN);
					else DATPORT &= ~(1 << DATPIN);

					CLKPORT |= (1 << CLKPIN);
					CLKPORT &= ~(1 << CLKPIN);
				}
			}
		#endif
		LATPORT |= (1 << LATPIN | 1 << BLANKPIN);
		LATPORT &= ~(1 << LATPIN | 1 << BLANKPIN);
	
		SPI_Used=0;		
	}		

}


//////////////////////////////////////////////////////////////////////////
//	Function: 	WriteLEDArray1
//	Parameters: count: Number of canvas PCBs to write
//              toggle_xlat: select if toggle xlat
//	Return:		None
//
//	Note:       The PCB canvas are written clock wise and from bottom to top
//
//////////////////////////////////////////////////////////////////////////
void WriteLEDArray1(uint8_t count, uint8_t toggle_xlat) 
{
	int16_t i=0;
	
	if(SPI_Used==0)
	{
		SPI_Used=1;
	
		#ifdef TLC5947_HARDWARE_SPI  //Hardware SPI
			uint8_t byte1;
			for (i = (24*count-1); i>0 ; i -= 2)
			{
				SPDR = LEDChannels[i] >> 4;
				byte1=((LEDChannels[i] & 0x000f) << 4) | (LEDChannels[i-1]>> 8);
				while(!(SPSR &(1<<SPIF)));
				SPDR= byte1;
				byte1= (uint8_t)LEDChannels[i-1];
				while(!(SPSR &(1<<SPIF)));
				SPDR = byte1;
				while(!(SPSR &(1<<SPIF)));
			}
		#else //Software SPI
			uint16_t tempOne = 0;
			for(i = (count*24);i>0;i--)
			{
				tempOne = LEDChannels[i-1];
				for (int j = 0; j < 12; j++)
				{
					if ((tempOne >> (11 - j)) & 1) DATPORT |= (1 << DATPIN);
					else DATPORT &= ~(1 << DATPIN);

					CLKPORT |= (1 << CLKPIN);
					CLKPORT &= ~(1 << CLKPIN);
				}
			}		
		#endif
		if(toggle_xlat)
		{
			Blank_ON();
			ToggleXLAT();
			Blank_OFF();
			//BLANK_ON();
			//TOGGLE_XLAT();
			//BLANK_OFF();
		}
	
		SPI_Used=0;		
	}

}


void ToggleXLAT()
{
	LATPORT |= (1 << LATPIN );
	LATPORT &= ~(1 << LATPIN );
}

void Blank_ON()
{
	LATPORT |= (1 << BLANKPIN);
}

void Blank_OFF()
{
	LATPORT &= ~(1 << BLANKPIN);
}

//------------------------------------------------------------------------------------------
// Sample function to draw a scanning pattern with fading
//------------------------------------------------------------------------------------------
void LEDscan(int red, float degreeoffset,unsigned int count) 
{

  float brightnessfactor = 0.0f;
  
  float scanindex = (1.0f + sinf(degreeoffset*3.14159f/180.0f)) * ((float)(NUM_TLC5947 * 24 ) / 2.0);
  
    for(unsigned int LEDindex = 0; LEDindex < (NUM_TLC5947  * 24); LEDindex++) {
      

      brightnessfactor = expf(0.0f - fabs(scanindex - ((float)LEDindex + 0.5f)) * 1.3f);

      LEDChannels[LEDindex] = red * brightnessfactor;
    }
    
    WriteLEDArray(count);  
}

void LEDscan2(int red, float degreeoffset,unsigned int count) 
{

  float brightnessfactor = 0.0f;
  
  float scanindex = (1.0f + sinf(degreeoffset*3.14159f/180.0f)) * ((float)(count * 24 ) / 2.0);
  
    for(unsigned int LEDindex = 0; LEDindex < (count  * 24); LEDindex++) {
      

      brightnessfactor = expf(0.0f - fabs(scanindex - ((float)LEDindex + 0.5f)) * 1.3f);

      LEDChannels[LEDindex] = red * brightnessfactor;
    }
}

void LEDscan2Add(int red, float degreeoffset,unsigned int count) 
{

  float brightnessfactor = 0.0f;
  
  float scanindex = (1.0f + sinf(degreeoffset*3.14159f/180.0f)) * ((float)(count * 24 ) / 2.0);
  
    for(unsigned int LEDindex = 0; LEDindex < (count  * 24); LEDindex++) {
      

      brightnessfactor = expf(0.0f - fabs(scanindex - ((float)LEDindex + 0.5f)) * 1.3f);

      LEDChannels[LEDindex] += red * brightnessfactor;

	  if( LEDChannels[LEDindex] > 4095 )  LEDChannels[LEDindex] = 4095;
    }
}


void LEDscan3(int red, float degreeoffset,unsigned int count) 
{

  float brightnessfactor = 0.0f;
  
  float scanindex = (1.0f + sinf(degreeoffset*3.14159f/180.0f)) * ((float)(count * 24 ) / 2.0);
  
    for(unsigned int LEDindex = 0; LEDindex < (count  * 24); LEDindex++) {
      

      brightnessfactor = expf(0.0f - fabs(scanindex - ((float)LEDindex + 0.5f)) * 1.3f);

      LEDChannels[LEDindex] = 4096-(red * brightnessfactor);
    }    
}



// from
// http://www.gammon.com.au/forum/?id=11497

void sleep_down( void )
{
	//set_sleep_mode (SLEEP_MODE_PWR_DOWN);  
	//sleep_enable();
//
	//// turn off brown-out enable in software
	//MCUCR = _BV (BODS) | _BV (BODSE);
	//MCUCR = _BV (BODS); 
	//sleep_cpu ();  

}

