/*
 * TLC5947.h
 *
 * Created: 11.01.2013 17:31:44
 *  Author: admin
 */ 


#ifndef TLC5947_H_
#define TLC5947_H_

#include <avr/io.h>

#define SUPERBALL_WIDTH			(24)
#define SUPERBALL_HEIGHT		(16)

#define SUPERBALL_COLUMNS        (16)
#define SUPERBALL_LED_PER_COLUMN (24)
#define NUM_TLC5947 ( 16 )

#define bit_set(reg, bit) reg |= 1 << bit
#define bit_clr(reg, bit) reg &= ~(1 << bit)

#define SWSPI_PORT PORTD
#define SWSPI_DIR  DDRD
#define SWSPI_IN   PIND

#ifndef TWO_PI
	#define TWO_PI 				( 6.283185307f )
#endif

#define DATPORT PORTC
#define CLKPORT PORTC //pc1
#define LATPORT PORTC
#define DATDDR DDRC

#define DATPIN PC0	//	SIN
#define CLKPIN PC1
#define BLANKPIN PC2
#define LATPIN PC3

#define SIN_PIN		_BV( PC0 ) /// PC0
#define SCLK_PIN	_BV( PC1 ) /// PC1
#define BLANK_PIN	_BV( PC2 )    /// PC2
#define XLAT_PIN	_BV( PC3 ) /// PC3

#define SWSPI_SS   (1<<1) // (CSn of the SPI can also be connected to GND permanently.)
#define SWSPI_SCK  (1<<2)  // CLK  SCL/SPC clock signal
#define SWSPI_MOSI (1<<4)  // mosi sda/sdi/sdo master out,slave in PD2
#define SWSPI_MISO (1<<4)  // miso SDO

#define TOGGLE_XLAT()  ({LATPORT|=(1 << LATPIN );\
	                     LATPORT &= ~(1 << LATPIN );})
					  
#define BLANK_ON()     (LATPORT|=(1<<BLANKPIN))	

#define BLANK_OFF()    (LATPORT&=~(1<<BLANKPIN))				  


extern uint16_t LEDChannels[(NUM_TLC5947*24)];
extern unsigned char brightnessShift;
extern volatile uint16_t brightness;
extern int gCount;

// prototypes

void SetPoint( uint8_t column, uint8_t row_led,uint16_t gray_scale)__attribute__((always_inline));
void SetPoint1 (uint8_t column, uint8_t row_led, uint16_t gray_scale, uint8_t clear_on, uint8_t enable_write)__attribute__((always_inline));
void SetColumnFullOn(uint8_t column, uint16_t gray_scale, uint8_t clear_on, uint8_t enable_write);
void SetColumnLedRangeOn(uint8_t column, uint8_t led_start, uint8_t led_end, uint16_t gray_scale, uint8_t clear_on, uint8_t enable_write);
void SetRowFullOn(uint8_t row, uint16_t gray_scale, uint8_t clear_on, uint8_t enable_write);
void SetRowLedOn(uint8_t row, uint8_t led, uint16_t gray_scale, uint8_t clear_on, uint8_t enable_write);
void SetColumnLedOn(uint8_t column, uint8_t led, uint16_t gray_scale, uint8_t clear_on, uint8_t enable_write);
void Clear(uint16_t val) __attribute__((always_inline));
void Line( int x,int y,int x1,int y1);
void sleep_down( void );

// TLC
void WriteLEDArray(uint8_t count);
void WriteLEDArray1(uint8_t count, uint8_t toggle_xlat) ;
void ToggleXLAT()__attribute__((always_inline));
void Blank_ON()__attribute__((always_inline));
void Blank_OFF()__attribute__((always_inline));
void LEDscan(int red, float degreeoffset,unsigned int count);
void LEDscan2(int red, float degreeoffset,unsigned int count);
void LEDscan3(int red, float degreeoffset,unsigned int count);
void LED_Init(uint16_t initial_grayscale);
void LEDscan2Add(int red, float degreeoffset,unsigned int count);

//sw spi
void SWSPI_init();
void SWSPI_write( uint8_t address, uint8_t value );
uint8_t SWSPI_read( uint8_t address );


#endif /* TLC5947_H_ */