/*
 * Timer.c
 *
 * Created: 21.07.2012 14:52:51
 *  Author: admin
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include "Timer.h"
#include "ADC.h"
#include "Goertzel.h"
//#include "Switch.h"
#include "settings.h"

volatile uint8_t tim1_irq=0;
volatile uint8_t tim1_voltage_cnt=0;
volatile uint8_t tim1_mode_debounce=1;
volatile uint8_t tim1_mode_switch=0;
volatile uint8_t tim1_brightness_debounce=1;
volatile uint8_t tim1_brightness_switch=0;
volatile uint8_t tim1_delay_done=1;
volatile uint8_t tim1_delay=1;
volatile uint8_t tim1_delay_cnt=0;
volatile uint16_t tim1_auto_delay_cnt=0;
volatile uint8_t tim1_auto_delay=1;
extern volatile uint16_t a_brightness[];

void Timer0_Init()
{
	//OP-Mode: CTC
	TCCR0A = (0<<COM0A1)|(0<<COM0A0)|(0<<COM0B1)|(0<<COM0B0)|(1<<WGM01)|(0<<WGM00);
	//Timer stopped (no clk-source)
	TCCR0B = (0<<FOC0A)|(0<<FOC0B)|(0<<WGM02)|eTimer0_PRESCALER_0;
	//Enable Compare Match IRQ
	TIMSK0 = (1<<OCIE0A);


}

void Timer0_Start(eTimer0_Frequence frequence)
{
	switch(frequence)
	{
		case eTimer0_Frequence_4KHz:
			//CPU-Clk=16MHz, prescaler=64, sample freuquency= 4 kHz
			OCR0A = 62; //Compare Value		
		break;
		case eTimer0_Frequence_8KHz:
			//CPU-Clk=16MHz, prescaler=64, sample freuquency= 4 kHz
			OCR0A = 30; //Compare Value
		break;
		//default frequence 4KHz
		default: OCR0A=62; break;
	}
	//Start Timer
	TCCR0B |= (0<<CS02)|(1<<CS01)|(1<<CS00);
}

void Timer0_Stop()
{
	//Stop timer 
	TCCR0B &= ~((1<<CS02)|(1<<CS01)|(1<<CS00));
}


ISR(TIMER0_COMPA_vect)
{	
	
	if(Goertzel_Enable)
	{
			//Read ADC Samples and fill buffer for Goertzel
			if(Goertzel_Count<NUMBER_OF_SAMPLES)
			{
				//start adc conversion
			   ADC_StartConversion(); //ADCSRA |= (1<<ADSC);
			   //Using assembler is much faster
			   //asm(
			   //"lds r16, 0x7A \n\t"
			   //"sbr r16, 64 \n\t"
			   //"sts 0x7A, r16"
			   //::);
			   
			   //Wait for completion
			   while((ADCSRA&(1<<ADSC)))asm("nop");
			   //Using Assembler is much faster
			   //asm(
			   //"loop: lds r16, 0x7A \n\t"  // ADCSRA in R16
			   //"      andi r16, 64 \n\t"     //(1<<ADSC) check if Comversion complete
			   //"      brne loop"
			   //::);
			   //Get result
			   GoertzelBuffer[Goertzel_Count]= (((ADC&0x03FF)<<6) - 32768)>>8;

			   //Goertzel filter
			   Goertzel_Count++;	
			}
			else
			{
				GoertzelBufferFilled=1;
				Goertzel_Enable = 0;
				Goertzel_Count = 0;	
				//Goertzel_F100=GoertzelOutput();
				//Goertzel_Complete=1;
			}		
												  										
	}

}


void Timer1_Init()
{
	//OP-Mode: CTC
	TCCR1A = (0<<COM1A1)|(0<<COM1A0)|(0<<COM1B1)|(0<<COM1B0)|(0<<WGM11)|(0<<WGM10);
	//Timer stopped (no clk-source)
	TCCR1B = (0<<ICNC1)|(0<<ICES1)|(0<<WGM13)|(1<<WGM12)|eTimer0_PRESCALER_0;
	//Enable Compare Match IRQ
	TIMSK1 = (1<<OCIE1A);


}

void Timer1_Start(eTimer1_Frequence frequence)
{
	//CPU-Clk=16MHz, prescaler=1024, Count frequency = 15625Hz (64 us)
	OCR1A=20000;
	switch(frequence)
	{
		case eTimer1_Frequence_100Hz:
		//CPU-Clk=16MHz, prescaler=1024, sample freuquency=  100 Hz
		OCR1A = 156; //Compare Value
		break;
		case eTimer1_Frequence_50Hz:
		//CPU-Clk=16MHz, prescaler=1024, sample freuquency=  50 Hz
		OCR1A = 312; //Compare Value
		break;
		case eTimer1_Frequence_20Hz:
		//CPU-Clk=16MHz, prescaler=1024, sample freuquency=  20 Hz
		OCR1A = 781; //Compare Value
		break;
		case eTimer1_Frequence_10Hz:
		//CPU-Clk=16MHz, prescaler=1024, sample freuquency=  10 Hz
		OCR1A = 1562; //Compare Value
		break;
		case eTimer1_Frequence_4Hz:
		//CPU-Clk=16MHz, prescaler=1024, sample freuquency=  4 Hz
		OCR1A = 3906; //Compare Value
		break;
		case eTimer1_Frequence_2Hz:
		//CPU-Clk=16MHz, prescaler=1024, sample freuquency=  2 Hz
		OCR1A = 7813; //Compare Value
		break;
		case eTimer1_Frequence_1Hz:
		//CPU-Clk=16MHz, prescaler=1024, sample freuquency=  1 Hz
		OCR1A = 15625; //Compare Value
		break;
		case eTimer1_Frequence_0k5Hz:
		//CPU-Clk=16MHz, prescaler=1024, sample freuquency= 0.5 Hz
		OCR1A = 31250; //Compare Value
		break;
		case eTimer1_Frequence_0k33Hz:
		//CPU-Clk=16MHz, prescaler=1024, sample freuquency= 0.33 Hz
		OCR1A = 46875; //Compare Value
		break;
		case eTimer1_Frequence_0k25Hz:
		//CPU-Clk=16MHz, prescaler=1024, sample freuquency=  0.25 Hz
		OCR1A = 62500; //Compare Value
		break;		
		//default frequence 0.25 Hz
		default: OCR1A=62500; break;
	}
	//Start Timer prescaler 1024
	TCCR1B |= (1<<CS02)|(0<<CS01)|(1<<CS00);
}

void Timer1_Stop()
{
	//Stop timer
	TCCR1B &= ~((1<<CS02)|(1<<CS01)|(1<<CS00));
}

void Timer1_Delay(uint8_t value)
{
	while(tim1_delay_done==0)asm("nop");
	tim1_delay=value;
	tim1_delay_done=0;
	while(tim1_delay_done==0)asm("nop");
}


void Timer1_IRQ_Delay()
{
	while(tim1_delay_done==0)asm("nop");
	tim1_delay_done=0;
	while(tim1_delay_done==0)asm("nop");	
}

ISR(TIMER1_COMPA_vect)
{
	#if(1) //timer 1 irq
	if(tim1_irq==0)
	{
		if(++tim1_voltage_cnt>200)
		{
			tim1_irq=1;
			tim1_voltage_cnt=0;
		}		
	}
	else
	{
		tim1_voltage_cnt=0;
	}
	#endif //timer 1 irq
	
	#if(1) //Automode Delay timer
	if(tim1_auto_delay_done==0)
	{
		if(++tim1_auto_delay_cnt==tim1_auto_delay)
		{
			tim1_auto_delay_done=1;
			tim1_auto_delay_cnt=0;
		}
	}
	else
	{
		tim1_auto_delay_cnt=0;
	}
	#endif //Automode Delay timer
	
	#if(1) //Animation Delay timer
	if(tim1_delay_done==0)
	{
		if(++tim1_delay_cnt==tim1_delay)
		{
			tim1_delay_done=1;
			tim1_delay_cnt=0;
		}
	}
	else
	{
		tim1_delay_cnt=0;
	}
	#endif //Animation Delay timer
	
	#if(1) //Mode switch debouncer
	if(tim1_mode_debounce==0)
	{
		tim1_mode_switch++;
		if ( !(PIND & (1<<MODE_KEY_PIN)) && (tim1_mode_switch>100))
		{
			if(Setup_mode==SETUP_MODE_SPEED) Setup_mode=SETUP_MODE_DISABLE;
			else if(Setup_mode==SETUP_MODE_DISABLE) Setup_mode=SETUP_MODE_SPEED;
			tim1_mode_debounce=1;
			tim1_mode_switch=0;
		}
		else if ( (PIND & (1<<MODE_KEY_PIN)) && (tim1_mode_switch>5 && tim1_mode_switch<100) )
		{
			
			if(Setup_mode==SETUP_MODE_SPEED)
			{
				//delay for auto-toggle animations
				if(mode==MODE_AUTO)
				{
					tim1_auto_delay += TIM1_MAX_AUTO_DELAY_INC;
					if(tim1_auto_delay > TIM1_MAX_AUTO_DELAY)tim1_auto_delay=TIM1_MAX_AUTO_DELAY;
				}
				//delay for manual-toggle animations
				else
				{
					if(++tim1_delay > TIM1_MAX_DELAY)tim1_delay=TIM1_MAX_DELAY;
				}
				
			}
			else if(Setup_mode==SETUP_MODE_BRIGHTNESS)
			{
				brightness+=BRIGHTNESS_SETUP_SETP;
				if(brightness>=BRIGHTNESS_MAX)brightness=BRIGHTNESS_MAX;
			}
			else if(Setup_mode==SETUP_MODE_DISABLE)
			{
				mode++;
				if (mode == NUM_OF_ANIMATIONS) mode =0;
				
			}
			tim1_mode_debounce=1;
			tim1_mode_switch=0;
			//break running delay and switch mode
			tim1_delay_done=1;
			tim1_delay_cnt=0;
		}
	}
	else
	{
		tim1_mode_switch=0;
	}
	#endif //Mode switch debouncer
	
	#if(1) //Bightness switch debouncer
	if(tim1_brightness_debounce==0)
	{
		
		tim1_brightness_switch++;
		if ( !(PIND & (1<<BRIGHT_KEY_PIN)) && (tim1_brightness_switch>100))
		{
			if(Setup_mode==SETUP_MODE_BRIGHTNESS) Setup_mode=SETUP_MODE_DISABLE;
			else if(Setup_mode==SETUP_MODE_DISABLE) Setup_mode=SETUP_MODE_BRIGHTNESS;
			tim1_brightness_debounce=1;
			tim1_brightness_switch=0;
		}
		else if ( (PIND & (1<<BRIGHT_KEY_PIN)) && (tim1_brightness_switch>5 && tim1_brightness_switch<100) )
		{
			
			if(Setup_mode==SETUP_MODE_SPEED)
			{
				//delay for auto-toggle animations
				if(mode==MODE_AUTO)
				{	
					tim1_auto_delay -= TIM1_MAX_AUTO_DELAY_INC;
					if(tim1_auto_delay < TIM1_MIN_DELAY)tim1_auto_delay=TIM1_MIN_DELAY;
				}
				//delay for manual toggle animations
				else
				{
					if(--tim1_delay < TIM1_MIN_DELAY)tim1_delay=TIM1_MIN_DELAY;
				}
				
			}
			else if(Setup_mode==SETUP_MODE_BRIGHTNESS)
			{
				brightness-=BRIGHTNESS_SETUP_SETP;
				if(brightness<=BRIGHTNESS_MIN)brightness=BRIGHTNESS_MIN;
			}
			else if(Setup_mode==SETUP_MODE_DISABLE)
			{
				brightness+=500;
				if(brightness>BRIGHTNESS_MAX)brightness=BRIGHTNESS_MIN;
				a_brightness[mode]=brightness;

			}
				tim1_brightness_debounce=1;
				tim1_brightness_switch=0;			
		}		

	}
	else
	{
		tim1_brightness_switch=0;
	}
	#endif //Bightness switch debouncer
}

