/*
 * UART.c
 *
 * Created: 20.10.2012 11:19:17
 *  Author: admin
 */ 
#include <avr/io.h>
#include "UART.h"
#include <avr/interrupt.h>

static char String[]="P";
volatile char UART_FrameFinished=0;
volatile char UART_FrameBuffer[20];
volatile uint8_t cnt1=0;

volatile char TxBuffer[RX_BUFFER_SIZE];
volatile char *pTxBufferIn;
volatile char *pTxBufferOut;
volatile uint8_t TxBufferPos;
volatile uint8_t TxBufferCount;
volatile uint8_t TxBufferState;
volatile char RxBuffer[RX_BUFFER_SIZE];
volatile char *pRxBufferIn;
volatile char *pRxBufferOut;
volatile uint8_t RxBufferPos;

void USART_init1(uint16_t baudrate)
{
	UCSR0B = ~((1<<RXCIE0)|(1<<UDRE0)|(1<<RXEN0)|(1<<TXEN0));
	pRxBufferIn = &RxBuffer[0];
	pRxBufferOut= &RxBuffer[0];
	RxBufferPos=0;
	pTxBufferIn = &TxBuffer[0];
	pTxBufferOut = &TxBuffer[0];
	TxBufferPos=0;
	TxBufferCount=0;
	switch (baudrate)
	{
		case 1200:
			UBRR0H = 0x03;
			UBRR0L = 0x40;
			break;
		case 9600:
			UBRR0H = 0;
			UBRR0L = 103; 
			break;
		case 38400:
			UBRR0H = 0;
			UBRR0L = 25;		
			break;
		case 115200:
			UBRR0H = 0;
			UBRR0L = 8;
			break;			
			
		//default 9600 baud
		default:
			UBRR0H = 0;
			UBRR0L = 103;
			break;
	};
	//UBRR0H = (uint8_t)(BAUD_PRESCALLER>>8);
	//UBRR0L = (uint8_t)(BAUD_PRESCALLER);

	//UBRR0H = 0;
	//UBRR0L = 51;
	UCSR0C = (3<<UCSZ00);
	UCSR0B = (1<<RXCIE0)|(1<<RXEN0)|(1<<TXEN0);
	//UCSR0B = (1<<RXCIE0)|(1<<UDRE0)|(1<<RXEN0)|(1<<TXEN0);
	
}

void UART_Disable()
{
	//Disable Receiver and Transmitter
	UCSR0B = ~((1<<RXCIE0)|(1<<RXEN0)|(1<<TXEN0));
	//clear pending interrupts
	uint8_t intflags = UART0_STATUS;
}

void UART_WriteTxBuffer(char Byte)
{
	//Write One Byte into Buffer
	uint8_t cnt=0;
	
	//if( !(UCSR0B&(1<<UDRIE0)) )UCSR0B |= (1<<UDRIE0);
	
	//End of Buffer Reached
	if(pTxBufferIn == &TxBuffer[TX_BUFFER_SIZE-1])
	{
		//Set to start
		pTxBufferIn = &TxBuffer[0];
	}
	//Buffer Empty
	if(pTxBufferIn == pTxBufferOut && TxBufferCount == 0)
	{
		*pTxBufferIn=Byte;
		pTxBufferIn++;
		TxBufferCount++;
		TxBufferState=TX_BUFFER_EMPTY;
	}
	//Buffer Full
	else if( (pTxBufferIn != pTxBufferOut) && (TxBufferCount <= (TX_BUFFER_SIZE-1)) )
	{
		*pTxBufferIn=Byte;
		pTxBufferIn++;
		TxBufferCount++;
		TxBufferState=TX_BUFFER_FULL;
	}
	cnt=TxBufferCount;
	////Last Byte
	//if(pTxBufferIn == pTxBufferOut && TxBufferCount > 0 )
	//{
		//TxBufferState=TX_BUFFER_LASTBYTE;
	//}
	////Buffer not empty
	//if(pTxBufferIn != pTxBufferOut && TxBufferCount > 0 )
	//{
		//TxBufferState=TX_BUFFER_NOT_EMPTY;
	//}
	

}

char UART_ReadTxBuffer()
{
	char Byte;
	//End of Buffer Reached
	if(pTxBufferOut == &TxBuffer[TX_BUFFER_SIZE-1])
	{
		//Set to start
		pTxBufferOut = &TxBuffer[0];
	}
	//Buffer Empty
	if(pTxBufferIn == pTxBufferOut && TxBufferCount == 0)
	{
		UCSR0B &= ~(1<<UDRIE0);
	}
	//Buffer Full
	else if( (pTxBufferIn != pTxBufferOut) && (TxBufferCount > 0) )
	{
		Byte = *pTxBufferOut;
		*pTxBufferOut=0;
		pTxBufferOut++;
		TxBufferCount--;
	}

	return Byte;

}

void UART_SendByte(char Byte)
{
	UART_WriteTxBuffer(Byte);
	//if( !(UCSR0B&(1<<UDRIE0)) )UCSR0B |= (1<<UDRIE0);
	//if(TxBufferCount==0) UCSR0B |= (1<<UDRE0);
}

void UART_SendString(char *pString)
{	
	UCSR0B &= ~(1<<UDRIE0);
	
	while(*pString != '\0')
	{
		UART_SendByte(*pString);
		pString++;
	}
	
	UCSR0B |= (1<<UDRIE0);
	//pString--;
	//WriteTxBuffer(0);
	//UDR0=0;
	
}

uint8_t UART_GetBufferFill()
{
	return TxBufferCount;
}

unsigned char USART_receive(void){

	while(!(UCSR0A & (1<<RXC0)));
	return UDR0;

}

void USART_send( unsigned char data){

	while(!(UCSR0A & (1<<UDRE0)));
	UDR0 = data;

}

void USART_putstring(char* StringPtr){

	while(*StringPtr != 0x00){
		USART_send(*StringPtr);
	StringPtr++;}

}


//////////////////////////////////////////////////////////////////////////
//Function: UART Tx finished interrupt
//Purpose:  called when the UART has received a character
//Data Received IRQ Handler
//////////////////////////////////////////////////////////////////////////
ISR(USART_TX_vect)
{
	asm("nop");
};

//////////////////////////////////////////////////////////////////////////
//Function: UART Data Register Empty interrupt
//Purpose:  called when the UART has received a character
//Data Received IRQ Handler
//////////////////////////////////////////////////////////////////////////
ISR(USART_UDRE_vect)
{
	char buf;
	buf=UCSR0A;
	buf=TxBufferCount;
	if(TxBufferCount >0 )
	{
		buf=UART_ReadTxBuffer();
		UART_FrameBuffer[cnt1++]=buf;
		UART0_DATA=buf;
	}
	if(TxBufferCount==0)
	{
		//UCSR0B &= ~(1<<UDRIE0);
	}		
	
};


//////////////////////////////////////////////////////////////////////////
//Function: UART Receive Complete interrupt
//Purpose:  called when the UART has received a character
//Data Received IRQ Handler
//////////////////////////////////////////////////////////////////////////
ISR(USART_RX_vect )
{
	static unsigned short data, lastData=0, userData=0 ;
	unsigned char usr;
	static char cnt;

	
	/* read UART status register and UART data register */
	usr  = UCSR0A;
	//data = UDR0;
	//UART_FrameBuffer[userData]=data;
	//if(userData++ > UART_MAX_DATA) userData=0;
	//if(userData>7)
	//{
		//data=0;
	//}

	
	//Fill Buffer
	if(RxBufferPos < RX_BUFFER_SIZE)
	{
		*pRxBufferIn=UART0_DATA;
		pRxBufferIn++;
		RxBufferPos++;
	}
	//On Overflow set to beginning
	if(RxBufferPos >= RX_BUFFER_SIZE)
	{
		RxBufferPos=0;
		pRxBufferIn = &RxBuffer[0];
		*pRxBufferIn=UART0_DATA;
	}





	//if(UART_FrameFinished==0)
	//{
		////UART_FrameFinished=1;
		////Wait for first Part of Start token
		//if(data == UART_STARTTOKEN && cnt==0) 
		//{
			//cnt=1;
			//UART_FrameFinished=0;
			//lastData=data;
			//userData=0;
		//}		
		////check if next byte is second byte of start token
		//else if(cnt==1 && lastData^data==0xff)
		//{
			//cnt=2;
		//}
		////if so, collect data
		//else if(cnt==2)
		//{
			//UART_FrameBuffer[userData]=data/12;
			//if(userData++ > UART_MAX_DATA)
			//{
				//cnt=0;
				//UART_FrameFinished=1;
				//lastData=0;
				//userData=0;
			//}
				//
		//}
		////if not, reset trigger logic
		//else cnt=0;
	//}


	//LEDChannels[indexLed++] = data *16;
//
//
	//// fill buffer
//
//
	//if( indexLed == (NUM_TLC5947*24) ) {
		//indexLed = 0;
		//pushLeds = 1;
//
	//}
}